#!/usr/bin/env bash

SCRIPT_PATH="$(
  cd "$(dirname "$0")" >/dev/null 2>&1 || exit
  pwd -P
)"
source "$SCRIPT_PATH"/common.sh # load commons

ENV_FILE="$SCRIPT_PATH"/$(basename "$0" .sh).env
LOG_FILE="$LOGS_DIR"/$(basename "$0" .sh).log

DOCKER_OS=centos
CMA_DOCKER_IMAGE_VERSION=latest
DOCKER_HOST_PORT=8090
DOCKER_CONTAINER_NAME=cma-server

DEFAULT_AM2CM_SERVER_PROFILE=prod
AM2CM_SERVER_PROFILE=${DEFAULT_AM2CM_SERVER_PROFILE}


for i in "$@"; do
  case $i in
  --start)
    ACTION_START=true
    ;;
  --stop)
    ACTION_STOP=true
    ;;
  --restart)
    ACTION_STOP=true
    ACTION_START=true
    ;;
  --rebuild)
    ACTION_REBUILD=true
    ;;
  -iv=* | --image-version=*)
    CMA_DOCKER_IMAGE_VERSION=${i#*=}
    ;;
  -P=* | --host-port=*)
    DOCKER_HOST_PORT=${i#*=}
    ;;
  -n=* | --container-name=*)
    DOCKER_CONTAINER_NAME=${i#*=}
    ;;
  -p=* | --profile=*)
    AM2CM_SERVER_PROFILE=${i#*=}
    ;;
  -h | --help)
    HELP=true
    ;;
  *)
    echo "Unknown option: $i"
    HELP=true
    ;;
  esac
done

__usage="
 Usage: $(basename "$0") [options]
 Options:
    --start: Start the container
    --stop: Stop the docker container
    --restart: Restart the docker container
    --rebuild: Rebuild the docker image
    -iv, --image-version: Set the version tag for the docker image ${CMA_DOCKER_IMAGE_VERSION})
    -P, --host-port: Set the host port to which the container port will be mapped (Default: ${DOCKER_HOST_PORT})
    -n, --container-name: Set the name of the container (Default: ${DOCKER_CONTAINER_NAME})
    -p, --profile: Set the active profile(s) for the am2cm-server (Default: $DEFAULT_AM2CM_SERVER_PROFILE)
    -h, --help: Print this help
 "

if [ "${HELP}" = true ]; then
  echo "$__usage"
  exit 0
fi

if [ -z "${ACTION_START}" -a -z "${ACTION_STOP}" -a -z "${ACTION_REBUILD}" ]; then
  ACTION_START=true
fi

echo "#################################################################################################################" >>"$LOG_FILE"
echo "Actions: ACTION_START: ${ACTION_START:-false}, ACTION_STOP: ${ACTION_STOP:-false}, ACTION_REBUILD: ${ACTION_REBUILD:-false}" >>"$LOG_FILE"

if ! docker info >/dev/null 2>&1; then
  echo "This script uses docker, and it isn't running - please start docker and try again!" | tee -a "$LOG_FILE"
  exit 1
fi

CMA_DOCKER_IMAGE_BASE=cma-${DOCKER_OS}
CMA_DOCKER_IMAGE=${CMA_DOCKER_IMAGE_BASE}:${CMA_DOCKER_IMAGE_VERSION}

stop_container() {
  if [ "$(docker ps -q -f name="${DOCKER_CONTAINER_NAME}")" ]; then
    echo -n "Stopping the ${DOCKER_CONTAINER_NAME} container... " | tee -a "${LOG_FILE}"
    docker stop "${DOCKER_CONTAINER_NAME}" >>"${LOG_FILE}" 2>&1

    if docker ps | grep "${DOCKER_CONTAINER_NAME}" >/dev/null 2>&1; then
      echo "Failed." | tee -a "$LOG_FILE"
      exit 1
    else
      echo "Done." | tee -a "$LOG_FILE"
    fi
  fi
}

delete_container() {
  if docker container inspect "${DOCKER_CONTAINER_NAME}" >/dev/null 2>&1; then

    echo -n "Deleting the ${DOCKER_CONTAINER_NAME} container... " | tee -a "$LOG_FILE"
    docker rm "${DOCKER_CONTAINER_NAME}" >>"$LOG_FILE" 2>&1

    if ! docker container inspect "${DOCKER_CONTAINER_NAME}" >/dev/null 2>&1; then
      echo "Done." | tee -a "$LOG_FILE"
    else
      echo "Failed." | tee -a "$LOG_FILE"
      exit 1
    fi
  fi
}

delete_image() {
  if docker image inspect "${CMA_DOCKER_IMAGE}" >/dev/null 2>&1; then

    echo -n "Deleting the ${CMA_DOCKER_IMAGE} image... " | tee -a "$LOG_FILE"
    docker rmi "${CMA_DOCKER_IMAGE}" >>"$LOG_FILE" 2>&1

    if ! docker image inspect "${CMA_DOCKER_IMAGE}" >/dev/null 2>&1; then
      echo "Done." | tee -a "$LOG_FILE"
    else
      echo "Failed." | tee -a "$LOG_FILE"
      exit 1
    fi
  fi
}

build_image() {
  if ! docker image inspect "${CMA_DOCKER_IMAGE}" >/dev/null 2>&1; then
    echo "The ${CMA_DOCKER_IMAGE} image is missing!" | tee -a "$LOG_FILE"
    "$SCRIPT_PATH"/setup.sh --docker -iv="${CMA_DOCKER_IMAGE_VERSION}" --profile=${AM2CM_SERVER_PROFILE}
    LAST_CMA_DOCKER_IMAGE="${CMA_DOCKER_IMAGE}"
  fi
}

rebuild_image() {
  if docker image inspect "${CMA_DOCKER_IMAGE}" >/dev/null 2>&1; then
    stop_container
    delete_container
    delete_image
    build_image
  fi
}

# get container ID and store in LAST_DOCKER_CONTAINER_ID variable
get_container_id() {
  docker ps -aq -f name=cma-server
}

run_container() {
  echo -n "Starting the ${DOCKER_CONTAINER_NAME} container... " | tee -a "$LOG_FILE"
  AM2CM_ROOT_DIR=/$(basename "${ROOT_DIR}")
  docker run -d --name "${DOCKER_CONTAINER_NAME}" -p "${DOCKER_HOST_PORT}":8090 -v "${ROOT_DIR}":"${AM2CM_ROOT_DIR}" "${CMA_DOCKER_IMAGE}" >>"$LOG_FILE" 2>&1
  LAST_DOCKER_CONTAINER_ID=$(get_container_id)
  if [ -z "${LAST_DOCKER_CONTAINER_ID}" ]; then
    echo "Failed."
  else
    echo "Done."
  fi
}

rebuild_container() {
  stop_container
  delete_container
  run_container
}

start_container() {
  docker start "${DOCKER_CONTAINER_NAME}" >>"$LOG_FILE" 2>&1
  LAST_DOCKER_CONTAINER_ID=$(get_container_id)
}

print_container_info() {
  if docker ps -q -f name="${DOCKER_CONTAINER_NAME}" >/dev/null 2>&1; then
    echo "Container-name: ${DOCKER_CONTAINER_NAME}" | tee -a "$LOG_FILE"
    echo "Container-id: ${LAST_DOCKER_CONTAINER_ID}" | tee -a "$LOG_FILE"
    echo "The am2cm-server will be available at http://localhost:${DOCKER_HOST_PORT} in about 30 seconds " | tee -a "$LOG_FILE"
  else
    echo "The container failed to start!" | tee -a "$LOG_FILE"
    exit 1
  fi
}

# storing the required variables in an .env file
write_env_file() {
  env_file=${1:-.env}
  cat >"$env_file" <<EOF
export LAST_CMA_DOCKER_IMAGE="${LAST_CMA_DOCKER_IMAGE}"
export LAST_DOCKER_CONTAINER_ID="${LAST_DOCKER_CONTAINER_ID}"
EOF
}

load_env_file "${ENV_FILE}"

if [ "${ACTION_STOP}" = true ]; then
  stop_container
fi

if [ "${ACTION_REBUILD}" = true ]; then
  rebuild_image
fi

# check if image exists otherwise generate one

if [ "${ACTION_START}" = true ]; then

  if ! docker image inspect "${CMA_DOCKER_IMAGE}" >/dev/null 2>&1; then
    build_image
  else
    if [ "${LAST_CMA_DOCKER_IMAGE}" != "${CMA_DOCKER_IMAGE}" ]; then
      while true; do
        if [ -z "${LAST_CMA_DOCKER_IMAGE}" ]; then
          BUILD_CMA_IMAGE=r
        else
          read -r -p "There is a ${CMA_DOCKER_IMAGE_BASE} image with the ${CMA_DOCKER_IMAGE_VERSION} tag. Do you want to [c]ontinue or [r]ebuild? [c]:" BUILD_CMA_IMAGE
        fi
        BUILD_CMA_IMAGE=${BUILD_CMA_IMAGE:-c}
        case $BUILD_CMA_IMAGE in
        [rR])
          rebuild_image
          break
          ;;
        [cC])
          echo "Continue with the existing image..."
          break
          ;;
        *) echo invalid response ;;
        esac
      done
    fi
  fi

  # check if $DOCKER_CONTAINER_NAME exists. Ask the user whether to continue or recreate it.
  if [ ! "$(docker ps -q -f name="${DOCKER_CONTAINER_NAME}")" ]; then
    if docker container inspect "${DOCKER_CONTAINER_NAME}" >/dev/null 2>&1; then
      while true; do
        if [ "${LAST_DOCKER_CONTAINER_ID}" != $(get_container_id) ]; then
          read -r -p "The container ${DOCKER_CONTAINER_NAME} exists. Do you want to [c]ontinue or [r]ebuild? [c]:" BUILD_CMA_CONTAINER
        fi
        BUILD_CMA_CONTAINER=${BUILD_CMA_CONTAINER:-c}
        case $BUILD_CMA_CONTAINER in
        [rR])
          echo "Rebuilding the ${DOCKER_CONTAINER_NAME} container..."
          rebuild_container
          break
          ;;
        [cC])
          echo "Continue with the existing container..."
          start_container
          break
          ;;
        *) echo invalid response ;;
        esac
      done
    else
      run_container
    fi
  fi

  print_container_info
fi

write_env_file "${ENV_FILE}"
