#!/usr/bin/env bash

SCRIPT_PATH="$(
  cd "$(dirname "$0")" >/dev/null 2>&1 || exit
  pwd -P
)"

source "$SCRIPT_PATH"/common.sh # load commons

LOG_FILE="$LOGS_DIR"/$(basename "$0" .sh).log

CMA_SERVER_HOME_DIR="${ROOT_DIR}/am2cm-server"
CMA_SERVER_PID_FILE="${CMA_SERVER_HOME_DIR}/am2cm-server.pid"

DEFAULT_CMA_SERVER_PROFILE=prod
CMA_SERVER_PROFILE="${DEFAULT_CMA_SERVER_PROFILE}"

SETUP_CMA_SERVER=false
VERBOSE=false

for i in "$@"; do
  case $i in
  --start)
    ACTION_START=true
    ;;
  --stop)
    ACTION_STOP=true
    ;;
  --restart)
    ACTION_STOP=true
    ACTION_START=true
    ;;
  --rebuild)
    ACTION_REBUILD=true
    ;;
  -p=* | --profile=*)
    CMA_SERVER_PROFILE=${i#*=}
    ;;
  -py=* | --python-executable=*)
    PYTHON_EXECUTABLE=${i#*=}
    ;;
  -h | --help)
    HELP=true
    ;;
  -v | --verbose)
    VERBOSE=true
    ;;
  *)
    echo "Unknown option: $i"
    HELP=true
    ;;
  esac
done

__usage="
 Usage: $(basename "$0") [options]
 Options:
    --start: Start the CMA server
    --stop: Stop the CMA server
    --restart: Restart the CMA server
    --rebuild: Reinstall the CMA server
    -p, --profile: Set the active profile(s) for the am2cm-server (Default: $DEFAULT_CMA_SERVER_PROFILE)
    -py, --python-executable: Path to python3 executable
    -v, --verbose: Print verbose log
    -h, --help: Print this help
 "

if [ "${HELP}" = true ]; then
  echo "$__usage"
  exit 0
fi

if [ -z "${ACTION_START}" -a -z "${ACTION_STOP}" -a -z "${ACTION_REBUILD}" ]; then
  ACTION_START=true
fi

echo "#################################################################################################################" >>"$LOG_FILE"
echo "Actions: ACTION_START: ${ACTION_START:-false}, ACTION_STOP: ${ACTION_STOP:-false}, ACTION_REBUILD: ${ACTION_REBUILD:-false}" >>"$LOG_FILE"

get_cma_server_pid() {
  if [ ! -f "${CMA_SERVER_PID_FILE}" ]; then
    echo ''
  else
    cat "${CMA_SERVER_PID_FILE}"
  fi
}

is_cma_server_running() {
  local pid=$(get_cma_server_pid)
  if [ -z "${pid}" -o "${pid}" = '' ]; then
    return 1
  fi
  kill -0 "$pid" >/dev/null 2>&1
}

cleanup() {
  echo -e '\n'
  if [ "${SETUP_CMA_SERVER}" = true ]; then
    if ! deleteVirtualEnv; then
      echo 'Next time the script should be called with the --rebuild option'
    fi
  fi
  echo 'The script is terminated'
  exit 1
}


execute_command() {
  if [ "${VERBOSE}" = true ]; then
    printf "\n" > /dev/tty
    "$@" | tee -a "$LOG_FILE"
  else
    "$@" >>"$LOG_FILE" 2>&1 &

    local pid=$!
    trap "kill ${pid}; cleanup" SIGINT

    while kill -0 ${pid} >/dev/null 2>&1; do
      printf '.' > /dev/tty
      sleep 2
    done
  fi

}

setup_cma_server() {

  if [ -z "${PYTHON_EXECUTABLE}" ]; then
    PYTHON_EXECUTABLE=$(get_python_executable)
  fi
  
  SETUP_CMA_SERVER=true

  echo -n "Installing the CMA server.." | tee -a "$LOG_FILE"
  execute_command "$SCRIPT_PATH"/setup.sh --local --profile="${CMA_SERVER_PROFILE}" --python-executable="${PYTHON_EXECUTABLE}"

  if [ -d "${VENV_DIR}" ]; then
    echo "Done." | tee -a "$LOG_FILE"
  else
    echo "Failed." | tee -a "$LOG_FILE"
    exit 1
  fi
}

stop_cma_server() {
  CMA_SERVER_PID=$(get_cma_server_pid)
  if is_cma_server_running; then
    echo -n "Stopping the CMA server.." | tee -a "$LOG_FILE"
    execute_command "$ROOT_DIR"/am2cm-server/am2cm-server.sh --stop

    # check if cma_server stopped
    sleep 3

    if ! is_cma_server_running; then
      echo "Done." | tee -a "$LOG_FILE"
    else
      echo "Failed." | tee -a "$LOG_FILE"
      exit 1
    fi
  fi
}

start_cma_server() {

  if ! is_cma_server_running; then
    echo -n "Starting the CMA server.." | tee -a "$LOG_FILE"
    execute_command "$ROOT_DIR"/am2cm-server/am2cm-server.sh --start --am2cm-root-dir="${ROOT_DIR}"


    # check if cma_server started
    if is_cma_server_running; then
      echo "Done." | tee -a "$LOG_FILE"
    else
      echo "Failed." | tee -a "$LOG_FILE"
      exit 1
    fi
  fi
}

reinstall() {
  # delete virtual env
  if ! deleteVirtualEnv; then
    exit 1
  fi
  setup_cma_server
}

if [ "${ACTION_STOP}" = true ]; then
  stop_cma_server
fi

if [ "${ACTION_REBUILD}" = true ]; then
  reinstall
fi

if [ "${ACTION_START}" = true ]; then
  if [ ! -d "${VENV_DIR}" ]; then
    setup_cma_server
    sleep 1
  fi
  start_cma_server
fi
